import onegov.onboarding

from morepath import setup
from onegov.core.utils import Bunch, scan_morepath_modules
from onegov.testing import utils
from onegov.town import TownApp
from webtest import TestApp as Client


def test_view_permissions():
    utils.assert_explicit_permissions(onegov.onboarding)


def test_default_assistant(onboarding_app):
    c = Client(onboarding_app)
    assert c.get('/').follow().request.url.endswith('for-towns/1')


def test_town_go_back(onboarding_app):
    c = Client(onboarding_app)
    a = c.get('/for-towns/1')

    a.form['name'] = 'New York'
    a.form['user'] = 'admin@example.org'
    a.form['color'] = '#ff00ff'
    a = a.form.submit().follow()

    assert 'New York' in a
    assert 'admin@example.org' in a
    assert '#f0f' in a
    assert 'new-york.example.org' in a

    a = a.click("Zurück")
    assert 'New York' in a
    assert 'admin@example.org' in a
    assert '#f0f' in a

    a.form['name'] = 'New Jersey'
    a.form['user'] = 'major@example.org'
    a = a.form.submit().follow()

    assert 'New Jersey' in a
    assert 'major@example.org' in a
    assert '#f0f' in a
    assert 'new-jersey.example.org' in a


def test_town_valid_values(onboarding_app):
    c = Client(onboarding_app)
    a = c.get('/for-towns/1')

    a.form['name'] = 'a' * 64
    a.form['user'] = 'admin'
    a.form['color'] = 'grüen'
    a = a.form.submit()

    assert "Feld kann nicht länger als 63 Zeichen sein" in a
    assert "Ungültige Email-Adresse" in a
    assert "'grüen' is not a recognized color" in a


def test_town_create(onboarding_app, temporary_directory, smtp):
    c = Client(onboarding_app)
    a = c.get('/for-towns/1')

    a.form['name'] = 'New York'
    a.form['user'] = 'admin@example.org'
    a.form['color'] = '#ff00ff'
    a = a.form.submit().follow()

    assert 'New York' in a
    assert 'admin@example.org' in a
    assert '#f0f' in a
    assert 'new-york.example.org' in a

    a = a.form.submit()

    assert 'https://new-york.example.org' in a
    assert 'admin@example.org' in a
    assert len(smtp.outbox) == 1

    username = 'admin@example.org'
    password = a.pyquery('.product dd:nth-child(4)').text()

    config = setup()
    scan_morepath_modules(onegov.town.TownApp, config)
    config.commit()

    town = TownApp()
    town.namespace = onboarding_app.onboarding['onegov.town']['namespace']
    town.configure_application(
        dsn=onboarding_app.dsn,
        filestorage='fs.osfs.OSFS',
        filestorage_options={
            'root_path': temporary_directory,
            'create': True
        },
        identity_secure=False,
        disable_memcached=True,
        enable_elasticsearch=False
    )
    town.set_application_id(town.namespace + '/' + 'new_york')
    town.registry.settings.cronjobs = Bunch(enabled=False)

    c = Client(town)
    p = c.get('/')

    assert "Willkommen bei OneGov" in p
    assert "New York" in p

    p = c.get('/auth/login')
    p.forms[1]['username'] = username
    p.forms[1]['password'] = password
    p = p.forms[1].submit().follow()

    assert 'Benutzerprofil' in p



from onegov.onboarding.models import Assistant


def test_assistant():

    class FooAssistant(Assistant):

        @Assistant.step()
        def first_step(self, request):
            return {'step': 1}

        @Assistant.step()
        def second_step(self, request):
            return {'step': 2}

        @Assistant.step()
        def third_step(self, request):
            return {'step': 3}

    foo = FooAssistant(None, current_step_number=1)
    assert foo.current_step.handle_view(None, None) == {'step': 1}
    assert foo.progress == (1, 3)
    assert foo.is_first_step == True
    assert foo.is_last_step == False

    foo = FooAssistant(None, current_step_number=2)
    assert foo.current_step.handle_view(None, None) == {'step': 2}
    assert foo.progress == (2, 3)
    assert foo.is_first_step == False
    assert foo.is_last_step == False

    foo = FooAssistant(None, current_step_number=3)
    assert foo.current_step.handle_view(None, None) == {'step': 3}
    assert foo.progress == (3, 3)
    assert foo.is_first_step == False
    assert foo.is_last_step == True



import logging
log = logging.getLogger('onegov.onboarding')  # noqa
log.addHandler(logging.NullHandler())  # noqa

from translationstring import TranslationStringFactory
_ = TranslationStringFactory('onegov.onboarding')  # noqa

from onegov.onboarding.app import OnboardingApp

__all__ = ['_', 'log', 'OnboardingApp']



from cached_property import cached_property
from hipchat import HipChat
from onegov.core import Framework, utils
from onegov.libres import LibresIntegration
from onegov.onboarding import log
from onegov.onboarding.theme import OnboardingTheme
from webassets import Bundle


class OnboardingApp(Framework, LibresIntegration):

    serve_static_files = True

    def configure_onboarding(self, **cfg):
        self.onboarding = cfg['onboarding']
        assert 'onegov.town' in self.onboarding
        assert 'namespace' in self.onboarding['onegov.town']

    def notify_hipchat(self, message):
        if 'hipchat' in self.onboarding:
            try:
                hipchat = HipChat(token=self.onboarding['hipchat']['token'])
                hipchat.message_room(
                    room_id=self.onboarding['hipchat']['room_id'],
                    message_from='Onboarding',
                    message=message,
                    message_format='html',
                    color='green',
                    notify=True
                )
            except Exception:
                log.exception("Error during Hipchat message")

    @cached_property
    def webassets_path(self):
        return utils.module_path('onegov.onboarding', 'assets')

    @cached_property
    def webassets_bundles(self):

        jsminifier = None

        common = Bundle(
            'js/modernizr.js',
            'js/jquery.js',
            'js/placeholder.js',
            'js/fastclick.js',
            'js/foundation.js',
            'js/underscore.js',
            'js/colorpicker.js',
            'js/awesomeplete.js',
            'js/common.js',
            filters=jsminifier,
            output='bundles/common.bundle.js'
        )

        return {
            'common': common
        }


@OnboardingApp.template_directory()
def get_template_directory():
    return 'templates'


@OnboardingApp.setting(section='core', name='theme')
def get_theme():
    return OnboardingTheme()


@OnboardingApp.setting(section='i18n', name='localedirs')
def get_i18n_localedirs():
    return [
        utils.module_path('onegov.onboarding', 'locale'),
        utils.module_path('onegov.form', 'locale'),
        utils.module_path('onegov.user', 'locale')
    ]


@OnboardingApp.setting(section='i18n', name='default_locale')
def get_i18n_default_locale():
    return 'de_CH'
